from .ecs import AliyunEcs
from .cs import AliyunCs
from .slb import AliyunSlb
from .vpc import AliyunVpc
from .dns import AliyunDns
from .ram import AliyunRam
from .bss import AliyunBss
from .rds import AliyunRds
from .redis import AliyunRedis
from .kms import AliyunKms
from .ons import AliyunOns
from .oss import AliyunOss
from .drds import AliyunDrds
from .pvtz import AliyunDnsPrivateZone
from .waf import AliyunWaf
from .log import AliyunLog
